#include "SafetyControllerFactory.h"

#include "ArmarXCore/core/exceptions/local/ExpressionException.h"

#include "RobotAPI/libraries/aron/core/navigator/data/primitive/String.h"
#include "Navigation/libraries/core/constants.h"
#include "Navigation/libraries/safety_control/LaserBasedProximity.h"
#include "Navigation/libraries/safety_control/SafetyController.h"
#include "Navigation/libraries/safety_control/core.h"


namespace armarx::nav::fac
{
    safe_ctrl::SafetyControllerPtr
    SafetyControllerFactory::create(const aron::datanavigator::DictNavigatorPtr& params,
                                    const core::Scene& ctx)
    {
        namespace layer = safe_ctrl;

        // algo name
        const auto algoName =
            aron::datanavigator::StringNavigator::DynamicCast(params->getElement(core::NAME_KEY));
        ARMARX_CHECK_NOT_NULL(algoName);
        const layer::Algorithms algo = layer::AlgorithmNames.from_name(algoName->getValue());

        // algo params
        const auto algoParams =
            aron::datanavigator::DictNavigator::DynamicCast(params->getElement(core::PARAMS_KEY));
        ARMARX_CHECK_NOT_NULL(algoParams);

        safe_ctrl::SafetyControllerPtr controller;
        switch (algo)
        {
            case safe_ctrl::Algorithms::LaserBasedProximity:
                controller = std::make_shared<safe_ctrl::LaserBasedProximity>(
                                 safe_ctrl::LaserBasedProximityParams::FromAron(algoParams), ctx);
                break;

        }

        return controller;
    }
} // namespace armarx::nav::fac
